@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,12 +20,20 @@
 #ifndef ZABBIX_TYPES_H
 #define ZABBIX_TYPES_H
 
+#include "sysinc.h"
+
 #if defined(_WINDOWS)
 #	define ZBX_THREAD_LOCAL __declspec(thread)
 #else
-#	if defined(HAVE_THREAD_LOCAL) && (defined(__GNUC__) || defined(__clang__) || defined(__MINGW32__))
-#		define ZBX_THREAD_LOCAL __thread
-#	else
+/* for non windows build thread local storage is required only for agent2 */
+#	if defined(ZBX_BUILD_AGENT2)
+#		if defined(HAVE_THREAD_LOCAL) && (defined(__GNUC__) || defined(__clang__) || defined(__MINGW32__))
+#			define ZBX_THREAD_LOCAL __thread
+#		else
+#			error "C compiler is not compatible with agent2 assembly"
+#		endif
+#	endif
+#	if !defined(ZBX_THREAD_LOCAL)
 #		define ZBX_THREAD_LOCAL
 #	endif
 #endif
@@ -159,8 +167,15 @@ typedef off_t	zbx_offset_t;
 
 #endif	/* _WINDOWS */
 
-#define	ZBX_FS_DBL		"%lf"
-#define	ZBX_FS_DBL_EXT(p)	"%." #p "lf"
+#define ZBX_FS_DBL		"%lf"
+#define ZBX_FS_DBL_EXT(p)	"%." #p "lf"
+#define ZBX_FS_DBL64		"%.17G"
+
+#ifdef HAVE_ORACLE
+#	define ZBX_FS_DBL64_SQL	ZBX_FS_DBL64 "d"
+#else
+#	define ZBX_FS_DBL64_SQL	ZBX_FS_DBL64
+#endif
 
 #define ZBX_PTR_SIZE		sizeof(void *)
 #define ZBX_FS_SIZE_T		ZBX_FS_UI64
