@@ -29,13 +29,13 @@ import (
 
 const (
 	APPLICATION_NAME        = "Zabbix Agent"
-	ZABBIX_REVDATE          = "28 February 2022"
+	ZABBIX_REVDATE          = "2 May 2022"
 	ZABBIX_VERSION_MAJOR    = 5
 	ZABBIX_VERSION_MINOR    = 0
-	ZABBIX_VERSION_PATCH    = 21
+	ZABBIX_VERSION_PATCH    = 23
 	ZABBIX_VERSION_RC       = ""
 	ZABBIX_VERSION_RC_NUM   = "2400"
-	ZABBIX_VERSION_REVISION = "47104dd574"
+	ZABBIX_VERSION_REVISION = "cbd238b4083"
 	copyrightMessage        = "Copyright (C) 2022 Zabbix SIA\n" +
 		"License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>.\n" +
 		"This is free software: you are free to change and redistribute it according to\n" +
