@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -146,18 +146,18 @@ extern "C" static int	parse_first_first(IEnumWbemClassObject *pEnumerator, doubl
 	if (WBEM_S_TIMEDOUT == hres)
 	{
 		*error = zbx_strdup(*error, "WMI query timeout.");
-		goto out;
+		goto out2;
 	}
 
 	if (FAILED(hres) || 0 == uReturn)
-		goto out;
+		goto out2;
 
 	hres = pclsObj->BeginEnumeration(WBEM_FLAG_NONSYSTEM_ONLY);
 
 	if (FAILED(hres))
 	{
 		*error = zbx_strdup(*error, "Cannot start WMI query result enumeration.");
-		goto out;
+		goto out1;
 	}
 
 	vtProp = (VARIANT*) zbx_malloc(NULL, sizeof(VARIANT));
@@ -168,7 +168,7 @@ extern "C" static int	parse_first_first(IEnumWbemClassObject *pEnumerator, doubl
 	{
 		*error = zbx_strdup(*error, "Cannot parse WMI result field.");
 		zbx_free(vtProp);
-		goto out;
+		goto out1;
 	}
 
 	pclsObj->EndEnumeration();
@@ -176,7 +176,7 @@ extern "C" static int	parse_first_first(IEnumWbemClassObject *pEnumerator, doubl
 	if (hres == WBEM_S_NO_MORE_DATA || VT_EMPTY == V_VT(vtProp) || VT_NULL == V_VT(vtProp))
 	{
 		zbx_free(vtProp);
-		goto out;
+		goto out1;
 	}
 	else
 		ret = SYSINFO_RET_OK;
@@ -187,10 +187,9 @@ extern "C" static int	parse_first_first(IEnumWbemClassObject *pEnumerator, doubl
 	zbx_vector_wmi_prop_create(inst_val);
 	zbx_vector_wmi_prop_append(inst_val, prop);
 	zbx_vector_wmi_instance_append(wmi_values, inst_val);
-out:
-	if (0 != pclsObj)
-		pclsObj->Release();
-
+out1:
+	pclsObj->Release();
+out2:	
 	return ret;
 }
 
@@ -213,15 +212,13 @@ extern "C" static int	parse_all(IEnumWbemClassObject *pEnumerator, double timeou
 		zbx_vector_wmi_instance_t *wmi_values, char **error)
 {
 	int	ret = SYSINFO_RET_FAIL;
-	VARIANT	*vtProp = NULL;
-	ULONG	obj_num = 0;
+	VARIANT	*vtProp;
 	HRESULT	hres = S_OK;
 
 	while (pEnumerator && SUCCEEDED(hres))
 	{
 		IWbemClassObject	*pclsObj;
 		ULONG			uReturn = 0;
-		HRESULT			hres;
 		zbx_vector_wmi_prop_t	*inst_val = NULL;
 
 		hres = pEnumerator->Next((long)(1000 * timeout), 1, &pclsObj, &uReturn);
@@ -233,6 +230,9 @@ extern "C" static int	parse_all(IEnumWbemClassObject *pEnumerator, double timeou
 			return ret;
 		}
 
+		if (WBEM_S_FALSE == hres && 0 == uReturn)
+			return SYSINFO_RET_OK;
+
 		if (FAILED(hres) || 0 == uReturn)
 			return ret;
 
@@ -245,15 +245,15 @@ extern "C" static int	parse_all(IEnumWbemClassObject *pEnumerator, double timeou
 			break;
 		}
 
-		inst_val = (zbx_vector_wmi_prop_t*) zbx_malloc(NULL, sizeof(zbx_vector_wmi_prop_t));
+		inst_val = (zbx_vector_wmi_prop_t*)zbx_malloc(NULL, sizeof(zbx_vector_wmi_prop_t));
 		zbx_vector_wmi_prop_create(inst_val);
 		zbx_vector_wmi_instance_append(wmi_values, inst_val);
 
-		while( !( FAILED(hres) || WBEM_S_NO_MORE_DATA == hres ))
+		while (!(FAILED(hres) || WBEM_S_NO_MORE_DATA == hres))
 		{
 			zbx_wmi_prop_t	prop = {NULL, NULL};
 
-			vtProp = (VARIANT*) zbx_malloc(NULL, sizeof(VARIANT));
+			vtProp = (VARIANT*)zbx_malloc(NULL, sizeof(VARIANT));
 			VariantInit(vtProp);
 			hres = pclsObj->Next(0, &prop.name, vtProp, 0, 0);
 
@@ -867,7 +867,7 @@ extern "C" int	convert_wmi_json(zbx_vector_wmi_instance_t *wmi_values, char **js
 	struct zbx_json	j;
 	int		inst_i, prop_i, ret = SYSINFO_RET_OK;
 
-	zbx_json_initarray(&j, wmi_values->values_num * wmi_values->values[0]->values_num * 50);
+	zbx_json_initarray(&j, ZBX_JSON_STAT_BUF_LEN);
 
 	for (inst_i = 0; inst_i < wmi_values->values_num && SYSINFO_RET_OK == ret; inst_i++)
 	{
