@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 package com.zabbix.gateway;
 
 import java.net.Socket;
+import java.util.Map;
+import java.util.Iterator;
 
 import org.json.*;
 
@@ -30,8 +32,11 @@ class SocketProcessor implements Runnable
 {
 	private static final Logger logger = LoggerFactory.getLogger(SocketProcessor.class);
 
+	private static long cleanupTime = System.currentTimeMillis();
 	private Socket socket;
 
+	public static final long MILLISECONDS_IN_HOUR = 1000 * 60 * 60;
+
 	SocketProcessor(Socket socket)
 	{
 		this.socket = socket;
@@ -52,9 +57,23 @@ class SocketProcessor implements Runnable
 			JSONObject request = new JSONObject(speaker.getRequest());
 
 			if (request.getString(ItemChecker.JSON_TAG_REQUEST).equals(ItemChecker.JSON_REQUEST_INTERNAL))
+			{
 				checker = new InternalItemChecker(request);
+			}
 			else if (request.getString(ItemChecker.JSON_TAG_REQUEST).equals(ItemChecker.JSON_REQUEST_JMX))
+			{
 				checker = new JMXItemChecker(request);
+
+				long now = System.currentTimeMillis();
+
+				if (now >= cleanupTime)
+				{
+					cleanDiscoveredObjects(now);
+					cleanupTime = now + MILLISECONDS_IN_HOUR;
+				}
+
+				cleanRMISSLhintCache((JMXItemChecker)checker);
+			}
 			else
 				throw new ZabbixException("bad request tag value: '%s'", request.getString(ItemChecker.JSON_TAG_REQUEST));
 
@@ -101,4 +120,34 @@ class SocketProcessor implements Runnable
 
 		logger.debug("finished processing incoming connection");
 	}
+
+	private void cleanDiscoveredObjects(long now)
+	{
+		for (Iterator<Map.Entry<String, Long>> it = JavaGateway.iterativeObjects.entrySet().iterator();
+			it.hasNext(); )
+		{
+			Map.Entry<String, Long> entry = it.next();
+			long expirationTime = entry.getValue();
+
+			if (now >= expirationTime)
+				it.remove();
+		}
+	}
+
+	public static final long MILLISECONDS_IN_24HOURS = MILLISECONDS_IN_HOUR * 24;
+	private static long RMISSLhintCacheCleanupTime = System.currentTimeMillis() + MILLISECONDS_IN_24HOURS;
+
+	public void cleanRMISSLhintCache(JMXItemChecker checker)
+	{
+		long now = System.currentTimeMillis();
+
+		logger.debug("RMI SSL hint cache cleanup is scheduled on " + RMISSLhintCacheCleanupTime +
+				", now is: " + now);
+
+		if (now >= RMISSLhintCacheCleanupTime)
+		{
+			checker.cleanUseRMISSLforURLHintCache();
+			RMISSLhintCacheCleanupTime = now + MILLISECONDS_IN_24HOURS;
+		}
+	}
 }
